/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.different.impl;

import java.util.ArrayList;
import java.util.List;

import org.tinygroup.different.DataWithOperation;

/**
 * 行对齐比较实现类<br>
 * 
 * 对比的List必须满足以下条件
 * 
 * 旧List与新List的长度一致(或为空)，两者的数据根据index一一对应
 * 
 * @author chenjiao
 * 
 * @param <T>
 */
public class LinePairDifferentImpl<T extends Comparable<T>> extends
		AbstractDifferentImpl<T> {

	public List<DataWithOperation<T>> different(List<T> list) {
		if (oldList == null) {// 如果是第一次
			oldList = list;
			return getNewList(list);
		}
		if (list == null) {
			return getRemoveList(oldList);
		}
		return compareList(list);
	}
	
	private List<DataWithOperation<T>> compareList(List<T> list) {
		if (oldList.size() != list.size()) {
			throw new RuntimeException("新列表与旧列表元素个数不相同！旧列表元素个数：["
					+ oldList.size() + "]，新列表元素个数：" + list.size());
		}
		List<DataWithOperation<T>> result = new ArrayList<DataWithOperation<T>>();
		for (int i = 0; i < oldList.size(); i++) {
			T oldObject = oldList.get(i);
			T newObject = list.get(i);
			if (oldObject.compareTo(newObject) != 0) {
				throw new RuntimeException("新列表与旧列表元素第[" + (i + 1)
						+ "]行主键主键不匹配！");
			}
			if (compare(oldObject, newObject) != 0) {
				result.add(new DataWithOperation<T>(DataWithOperation.MODIFY,
						newObject));
			}
		}
		oldList = list;
		return result;
	}

}
